import type { ComponentInstance, MiddlewareHandler, RouteData, RouteType, RuntimeMode, SSRLoadedRenderer, SSRManifest, SSRResult, AstroUserConfig } from '../@types/astro.js';
/**
 * Options to be passed when rendering a route
 */
export type ContainerRenderOptions = {
    /**
     * If your component renders slots, that's where you want to fill the slots.
     * A single slot should have the `default` field:
     * ```js
     * container.renderToString(Component, { slots: { default: "Some value"}});
     * ```
     *
     */
    slots?: Record<string, any>;
    /**
     * The request is used to understand which path/URL the component is about to render.
     *
     * Use this option in case your component or middleware needs to read information like `Astro.url` or `Astro.request`.
     */
    request?: Request;
    /**
     * Useful for dynamic routes. If your component is something like `src/pages/blog/[id]/[...slug]`, you'll want to provide:
     * ```js
     * container.renderToString(Component, { params: ["id", "...slug"] });
     * ```
     */
    params?: string[];
    /**
     * Useful if your component needs to access some locals without the use a middleware.
     * ```js
     * container.renderToString(Component, { locals: { getSomeValue() {} } });
     * ```
     */
    locals?: App.Locals;
    /**
     * Useful in case you're attempting to render an errored route.
     */
    status?: number;
    /**
     * Useful in case you're attempting to render an endpoint:
     * ```js
     * container.renderToString(Endpoint, { routeType: "endpoint" });
     * ```
     */
    routeType?: RouteType;
};
type AstroContainerOptions = {
    mode?: RuntimeMode;
    streaming?: boolean;
    renderers?: SSRLoadedRenderer[];
    astroConfig?: AstroUserConfig;
    middleware?: MiddlewareHandler;
    resolve?: SSRResult['resolve'];
    manifest?: SSRManifest;
};
export type AstroContainerManifest = Pick<SSRManifest, 'middleware' | 'clientDirectives' | 'inlinedScripts' | 'componentMetadata' | 'renderers' | 'assetsPrefix' | 'base' | 'routes' | 'assets' | 'entryModules' | 'compressHTML' | 'trailingSlash' | 'buildFormat' | 'i18n'>;
export declare class unstable_AstroContainer {
    #private;
    private constructor();
    containerResolve(specifier: string): Promise<string>;
    static create(containerOptions?: AstroContainerOptions): Promise<unstable_AstroContainer>;
    static createFromManifest(manifest: SSRManifest): Promise<unstable_AstroContainer>;
    /**
     * Use this method to manually insert a route inside the container.
     *
     * This method is useful if you're rending a component that attempt to render redirect or a rewrite.
     *
     * @param {object} options
     * @param {string} options.path The path of the route. It has to match what you see in a URL browser, e.g. `/blog/12334/first-post`
     * @param {ComponentInstance} options.componentInstance The a compile instance of the Astro component.
     * @param {string[]} options.params The params of the route. Use these when your route is dynamic. For a route `/blog/[id]/[...dynamic]`, the params are `["id", "...dynamic"]`
     * @param {RouteType} options.type The type of route.
     */
    insertRoute({ path, componentInstance, params, type, }: {
        path: string;
        componentInstance: ComponentInstance;
        params?: string[];
        type?: RouteType;
    }): RouteData;
    /**
     *
     * @param component The instance of the component.
     * @param options Some options.
     */
    renderToString(component: ComponentInstance, options?: ContainerRenderOptions): Promise<string>;
    renderToResponse(component: ComponentInstance, options?: ContainerRenderOptions): Promise<Response>;
    createRoute(url: URL, params: string[], type: RouteType): RouteData;
}
export {};
