import { TestPipeline } from "./pipeline.js";
import { Logger } from "../core/logger/core.js";
import { nodeLogDestination } from "../core/logger/node.js";
import { toRoutingStrategy } from "../i18n/utils.js";
import { validateConfig } from "../core/config/config.js";
import { ASTRO_CONFIG_DEFAULTS } from "../core/config/schema.js";
import { RenderContext } from "../core/render-context.js";
import { posix } from "node:path";
import { getParts, getPattern, validateSegment } from "../core/routing/manifest/create.js";
import { removeLeadingForwardSlash } from "../core/path.js";
function createContainerManifest(renderers, config, manifest, middleware) {
  let i18nManifest = void 0;
  if (config?.i18n) {
    i18nManifest = {
      fallback: config?.i18n.fallback,
      strategy: toRoutingStrategy(config?.i18n.routing, config?.i18n.domains),
      defaultLocale: config?.i18n.defaultLocale,
      locales: config?.i18n.locales,
      domainLookupTable: {}
    };
  }
  const defaultMiddleware = (_, next) => {
    return next();
  };
  return {
    rewritingEnabled: false,
    trailingSlash: manifest?.trailingSlash ?? config?.trailingSlash,
    buildFormat: manifest?.buildFormat ?? config?.build.format,
    compressHTML: manifest?.compressHTML ?? config?.compressHTML,
    assets: manifest?.assets ?? /* @__PURE__ */ new Set(),
    assetsPrefix: manifest?.assetsPrefix ?? void 0,
    entryModules: manifest?.entryModules ?? {},
    routes: manifest?.routes ?? [],
    adapterName: "",
    clientDirectives: manifest?.clientDirectives ?? /* @__PURE__ */ new Map(),
    renderers: manifest?.renderers ?? renderers,
    base: manifest?.base ?? config?.base,
    componentMetadata: manifest?.componentMetadata ?? /* @__PURE__ */ new Map(),
    inlinedScripts: manifest?.inlinedScripts ?? /* @__PURE__ */ new Map(),
    i18n: manifest?.i18n ?? i18nManifest,
    checkOrigin: false,
    middleware: manifest?.middleware ?? middleware ?? defaultMiddleware
  };
}
class unstable_AstroContainer {
  #pipeline;
  #config;
  /**
   * Internally used to check if the container was created with a manifest.
   * @private
   */
  #withManifest = false;
  constructor({
    streaming = false,
    renderers = [],
    config,
    manifest,
    resolve
  }) {
    this.#config = config;
    this.#pipeline = TestPipeline.create({
      logger: new Logger({
        level: "info",
        dest: nodeLogDestination
      }),
      manifest: createContainerManifest(renderers, config, manifest),
      streaming,
      serverLike: true,
      renderers,
      resolve: async (specifier) => {
        if (this.#withManifest) {
          return this.containerResolve(specifier);
        } else if (resolve) {
          return resolve(specifier);
        }
        return specifier;
      }
    });
  }
  async containerResolve(specifier) {
    const found = this.#pipeline.manifest.entryModules[specifier];
    if (found) {
      return new URL(found, this.#config.build.client).toString();
    }
    return found;
  }
  static async create(containerOptions = {}) {
    const {
      astroConfig = ASTRO_CONFIG_DEFAULTS,
      streaming = false,
      renderers = [],
      resolve,
      manifest
    } = containerOptions;
    const config = await validateConfig(astroConfig, process.cwd(), "container");
    return new unstable_AstroContainer({ streaming, renderers, config, manifest, resolve });
  }
  static async createFromManifest(manifest) {
    const config = await validateConfig(ASTRO_CONFIG_DEFAULTS, process.cwd(), "container");
    const container = new unstable_AstroContainer({
      manifest,
      config
    });
    container.#withManifest = true;
    return container;
  }
  /**
   * Use this method to manually insert a route inside the container.
   *
   * This method is useful if you're rending a component that attempt to render redirect or a rewrite.
   *
   * @param {object} options
   * @param {string} options.path The path of the route. It has to match what you see in a URL browser, e.g. `/blog/12334/first-post`
   * @param {ComponentInstance} options.componentInstance The a compile instance of the Astro component.
   * @param {string[]} options.params The params of the route. Use these when your route is dynamic. For a route `/blog/[id]/[...dynamic]`, the params are `["id", "...dynamic"]`
   * @param {RouteType} options.type The type of route.
   */
  insertRoute({
    path,
    componentInstance,
    params = [],
    type = "page"
  }) {
    const pathUrl = new URL(path, "https://example.com");
    const routeData = this.createRoute(pathUrl, params, type);
    this.#pipeline.manifest.routes.push({
      routeData,
      file: "",
      links: [],
      styles: [],
      scripts: []
    });
    this.#pipeline.insertRoute(routeData, componentInstance);
    return routeData;
  }
  /**
   *
   * @param component The instance of the component.
   * @param options Some options.
   */
  async renderToString(component, options = {}) {
    const response = await this.renderToResponse(component, options);
    return await response.text();
  }
  async renderToResponse(component, options = {}) {
    const { routeType = "page", slots } = options;
    const request = options?.request ?? new Request("https://example.com/");
    const params = options?.params ?? [];
    const url = new URL(request.url);
    const routeData = this.insertRoute({
      path: request.url,
      componentInstance: component,
      params,
      type: routeType
    });
    const renderContext = RenderContext.create({
      pipeline: this.#pipeline,
      routeData,
      status: options?.status ?? 200,
      middleware: this.#pipeline.middleware,
      request,
      pathname: url.pathname,
      locals: options?.locals ?? {}
    });
    return renderContext.render(component, slots);
  }
  createRoute(url, params, type) {
    const segments = removeLeadingForwardSlash(url.pathname).split(posix.sep).filter(Boolean).map((s) => {
      validateSegment(s);
      return getParts(s, url.pathname);
    });
    return {
      component: "",
      generate(_data) {
        return "";
      },
      params,
      pattern: getPattern(segments, this.#config, this.#config.trailingSlash),
      prerender: false,
      segments,
      type,
      route: url.pathname,
      fallbackRoutes: [],
      isIndex: false
    };
  }
}
export {
  unstable_AstroContainer
};
